<div class="header">
	<div class="row">
		<div class="col-sm-6"><h3>Members</h3></div>
		<div class="col-sm-6 dropdown">
			<input type="text" id="find-users" class="pull-right form-control" placeholder="Search for users to add to the group">
			<ul class="dropdown-menu type-away">
				<li role="presentation" class="dropdown-header">Click on item to add user</li>
			</ul>
		</div>
	</div>
</div>

{% if users.count %}
<ul class="user-list">
	{% for user in users %}
		<li>
			{{user.display_name}} 
			{% if users|length > 1 or not admins %}<a class="text-danger remove" data-user-id="{{ user.id }}" href="#"><small>remove</small></a>{% endif %}
		</li>
	{% empty %}
		<li>No users have been assigned to this group.</li>
	{% endfor %}
</ul>
{% else %}
<div class="well empty-list">
    <p class="lead">We couldn't find any users assigned to this group.</p>
    <p>Adding users to a group is easy: Type a name in the input field to the top right to find the user you want to add and then simply click on the name in the result list to add the user to this group.</p>
</div>
{% endif %}

