<div class="form-group">
    <label for="{{key}}-ref" class="col-sm-3 col-sm-offset-1 control-label">contains</label>
    <div class="col-sm-7">
        <input type="text" class="form-control" name="{{key}}-ref" id="reference-value" class="reference-value">
    </div>
</div>
