<div class="form-group">
    <label for="{{key}}-min" class="col-sm-3 col-sm-offset-1 control-label">is larger than</label>
    <div class="col-sm-7">
        <input type="number" class="form-control" data-key="{{key}}" name="{{key}}-min" id="{{key}}-min" {{#if minval}}placeholder="{{minval}}" {{/if}}>
    </div>
</div>

<div class="form-group">
    <label for="{{key}}-max" class="col-sm-3 col-sm-offset-1 control-label">is lower than</label>
    <div class="col-sm-7">
        <input type="number" class="form-control" data-key="{{key}}" name="{{key}}-max" id="{{key}}-max" {{#if maxval}}placeholder="{{maxval}}" {{/if}}>
    </div>
</div>
