html, body {
	margin: 0;
    height: 100%;
}

body {
	padding-bottom: 20px;
}

body#home {
	background: #373f45 url(../img/people.jpg) no-repeat 50% 50%;
	background-size: cover;
	height: 100%;
}

body#home div.jumbotron {
	background-color: transparent;
	padding: 10px 0;
}

	body#home div.jumbotron div.credit {
		margin-bottom: 90px;
		font-size: 12px;
	}

	body#home div.jumbotron div.credit a {
		color: #333;
		text-decoration: underline;
	}

	body#home div.jumbotron .super-heading {
		text-shadow: 0 0 10px #555;
		color: #fff;
	}

	body#home div.jumbotron h1 {
		margin-top: 0;
	}

	body#home div.jumbotron .footer {
		font-size: 14px;
		font-weight: normal;
	}

	body#home div.jumbotron .footer .panel-default {
		text-align: center;
		background-color: rgba(255,255,255,0.7);
		border: none;
	}

	body#home div.jumbotron .footer .panel-default .panel-body {
		padding: 5px;
	}

body.login {
	padding-top: 90px;
}

body.login div.page-header {
	margin-top: -40px;
}

.alert ul {
	margin-left: 0;
	padding-left: 25px;
}

.row {
	margin-bottom: 30px;
}

.loading {
	background: url(/static/img/ajax-loader.gif) no-repeat;
	background-position: right 10px top 50%;
}

.info-loading {
	background-image: url(/static/img/ajax-loader-blue.gif);
	background-repeat: no-repeat;
	background-position: left 10px top 50%;
	padding-left: 35px;
}

.panel-heading.loading {
	background-position: right 10px top 10px;
}

.success {
	background: url(/static/img/success.png) no-repeat;
	background-position: right 10px top 50%;
}

.panel-heading.success {
	background-position: right 10px top 10px;
}

#descriptionText.success,
.checkbox.success {
	padding-right: 35px;
}

.hidden.radio+.radio {
	margin-top: 10px;
}

h1:first-child,
h2:first-child,
h3:first-child,
h4:first-child,
h5:first-child,
h6:first-child {
	margin-top: 0;
}

h1.smaller {
	font-size: 24px;
	font-weight: normal;
}

h1.smaller a:link,
h1.smaller a:visited {
	color: rgb(51, 51, 51);
}

h1.smaller a:hover,
h1.smaller a:focus {
	color: rgb(66, 139, 202);
	text-decoration: none;
}

.navbar-default {
	background-color: #f5f5f5;
}

div.page-header {
	position: relative;
	padding-top: 20px;
	padding-bottom: 20px;
	margin-bottom: 30px;
	background-color: #fafafa;
}

	div.page-header p.lead {
		max-height: 4em;
		overflow: auto;
	}

	div.page-header div.row {
		margin-bottom: 0;
	}

div.page-header .breadcrumb {
	background-color: #fafafa;
	border-bottom: 1px solid #eee;
	padding-left: 0;
	padding-right: 0;
	margin: 0;
}

div.page-header .breadcrumb > li + li:before {
	content: "\00BB";
}

	div.page-header h1 {
		margin-top: 20px;
		margin-bottom: 5px;
	}

	div.page-header p.lead {
		margin-bottom: 0;
	}

	div.page-header .form-inline {
		margin-top: 0;
		margin-bottom: 0;
	}
		div.page-header .form-inline > .form-group {
			width: 100%;
		}

		@media (min-width: 768px) {
			div.page-header .form-inline > .form-group {
				width: 583px;
			}
			div.page-header .form-inline > .form-group > input[type="text"] {
				width: 100%;
			}
		}

		div.page-header .form-inline p.error {
			margin-top: 10px;
		}

	div.page-header > .col-sm-12 {
		padding-left: 0;
		padding-right: 0;
	}

		div.page-header > .col-sm-12 > .alert {
			margin: 5px 0 0 0;
		}

.panel-title + p {
	margin-top: 0.5em;
}

ul.type-away {
	width: 280px;
	margin-left: 20px;
	border-top-left-radius: 0;
	border-top-right-radius: 0;
	border-top: 0;
}

.list-group-item.active a {
	color: #fff;
}

.bordered-panel .tab-pane {
	padding: 15px;
	border: 0 solid #ddd;
	border-width: 0 1px 1px 1px;
	border-bottom-left-radius: 3px;
	border-bottom-right-radius: 3px;
}

.field-filter {
	padding: 15px;
	border-bottom: 1px solid #e7e7e7;
	font-weight: bold;
}

.field-filter label,
.field-filter input,
.field-filter select {
	font-weight: normal;
}

 .field-filter input[type="number"] {
 	width: 100px;
 }

footer {
	text-align: center;
	padding-top: 30px;
	padding-bottom: 30px;
	border-top: 1px solid #eee;
	background-color: #fafafa;
}

footer p {
	margin: 0;
}

.settings-link {
	padding-top: 25px;
}

div.well.empty-list {
	margin-top: 2em;
}

ul.overview-list > li,
ol.overview-list > li {
  padding: 1.5em 0;
  border-top: 1px solid #ddd;
}

ul.overview-list > li:first-child,
ol.overview-list > li:first-child {
  padding-top: 0;
  border-top: none;
}

ul.overview-list > li:last-child,
ol.overview-list > li:last-child {
  padding-bottom: 0;
}

ul.overview-list > li .checkbox {
	padding-left: 20px;
	margin-bottom: 0;
	margin-top: 0;
	padding-top: 15px;
	padding-bottom: 15px;
}

	ul.overview-list > li h3:last-child,
	ul.overview-list > li p:last-child {
		margin-bottom: 0;
	}

	ul.overview-list > li a + h3 {
		margin-top: 0;
	}

	ul.overview-list > li h3 a {
		text-decoration: none;
	}

	ul.overview-list > li h3 span.label {
		font-size: 0.5em;
	}

	ul.overview-list > li p.meta {
		color: #999;
		margin-bottom: 0;
		text-transform: uppercase;
		font-size: 12px;
	}

	ul.overview-list > li p.meta span.lower-case {
		text-transform: none;
	}

	ul.overview-list > li.message {
		margin-top: -15px;
		margin-bottom: 15px;
		padding: 15px;
	}

	ul.overview-list > li.ui-sortable-helper {
		border-bottom: 1px solid #ddd;
	}

li.ui-state-highlight {
	background-color: #fcf8e3;
	border: 1px dashed #ddd;
	height: 50px;
	margin: 15px 0;
}

ul.overview-list > li.sort-item span.glyphicon:hover {
	cursor: move;
}

ul.overview-list > li.sort-item p {
	padding-left: 22px;
}

ul.overview-list > li > div:last-child {
	margin-bottom: 0;
}

.app-meta code {
	display: block;
	word-break: break-all;
	overflow-wrap: break-word;     /* Renamed property in CSS3 draft spec */
	width: 100%;
}

.becareful {
	padding: 0;
}

.becareful > div {
	border-bottom: 1px solid #ddd;
	padding: 15px;
}

.becareful > div:last-child {
	border-bottom: 0;
}

.page-header .row {
	position: relative;
}

.page-header .header-nav {
	position: absolute;
	bottom: -33px;
	right: 0;
}

.page-header .header-nav li {
	list-style: none;
	display: inline-block;
	padding: 10px;
	margin-left: 15px;
	border-bottom: 3px solid transparent;
}

.page-header .header-nav li.active {
	border-bottom-color: #5bc0de;
}

.header {
	border-bottom: 1px solid #ddd;
	padding-bottom: 12px;
	margin-bottom: 24px;
}

.header .row {
	margin-bottom: 0;
}

#grouping-visibilty .message {
	margin: 0 -15px 15px -15px;
	padding: 15px;
}

.user-list {
	list-style: none;
	margin: 0;
	padding: 0;
}

.user-list li {
	margin: 0;
	padding: 15px 0;
	border-bottom: 1px solid #ddd;
}

.user-list li.message {
	padding: 15px;
}

.user-list li:first-child {
	margin-top: -12px;
}

.user-list li:last-child {
	border-bottom: none;
}

fieldset {
	margin-top: 30px;
}

fieldset legend {
	margin-bottom: 10px;
}

form#permissions {
	margin-bottom: 60px;
}

form#permissions .message {
	padding: 15px;
}

#stats, .contributions {
	color: #428bca;
	text-transform: uppercase;
	font-weight: bold;
	text-align: center;
}

.category {
	clear: both;
	border-bottom: 1px solid #ddd;
	padding-top: 10px;
	padding-bottom: 10px;
}

.field-options {
	margin-top: 15px;
	margin-bottom: 15px;
}

.number {
	font-weight: bold;
	line-height: 1;
	display: inline-block;
	color: #fff;
	background-color: #428bca;
}

.contributions {
	text-align: left;
}

	#stats .number {
		font-size: 4em;
		min-width: 86px;
		padding: 15px;
		border-radius: 43px;
		margin-bottom: 15px;
	}

	.contributions .number {
		text-align: center;
		font-size: 1em;
		min-width: 28px;
		padding: 7px;
		border-radius: 14px;
	}

.panel-heading h3:last-child,
.panel-heading p,
.panel-body p:last-child {
	margin-bottom: 0;
}

h6.item-info {
	text-transform: uppercase;
	margin-bottom: 6px;
	font-size: 12px;
	color: #777;
}

h6.item-info + h3 {
	margin-top: 0;
}

.field-info {
	table-layout: fixed;
}

.field-info td:first-child {
	padding-left: 15px;
	width: 80px;
}

.field-info td:last-child {
	padding-right: 15px;

}

.field-info td code {
	width: 100%;
	display: block;
    overflow: auto;
    white-space: nowrap;
}

#help-block h4 {
	padding-left: 40px;
	text-indent: -40px;
}

#help-block h4 span.step {
	color: #fff;
	background-color: #428bca;
	display: inline-block;
	width: 34px;
	line-height: 34px;
	border-radius: 17px;
	text-align: center;
	text-indent: 0;
}

#help-block p + h4 {
	margin-top: 40px;
}

#help-block p {
	padding-left: 40px;
}

.message {
	padding: 15px;
	margin-bottom: 15px;
}

#map {
	height: 400px;
	margin-bottom: 30px;
}
