# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('categories', '0007_auto_20150130_1155'),
    ]

    operations = [
        # migrations.AlterField(
        #     model_name='category',
        #     name='display_field',
        #     field=models.ForeignKey(related_name='display_field_of', to='categories.Field', null=True),
        #     preserve_default=True,
        # ),
    ]
