{% extends "base.html" %}

{% block title %} | Reset password{% endblock %}
{% block bodydata %} class="login" {% endblock %}

{% block main %}
        <div class="container">
            <div class="row">
                <div class="col-sm-8 col-sm-offset-2 col-md-6 col-md-offset-3">
                    <h3 class="header">Reset Password</h3>
                    <form role="form" method="post" action="{% url 'account_reset_password' %}" novalidate>
                        {% csrf_token %}
                        <div class="form-group">
                            <label for="email">Please specify your email address to receive instructions for resetting it.</label>
                            <input type="email" class="form-control input-lg" name="email" id="email" required>
                        </div>
                        <div class="form-group">
                            <button type="submit" class="btn btn-primary">Reset password</button>
                        </div>
                    </form>
                    <p class="text-center">Not a member yet? Please <a href="{% url 'account_signup' %}">sign up here</a>
                </div>
            </div>
        </div>
{% endblock %}
