<div id="help-block" class="col-sm-8 hidden">
    <h3 class="header">Getting started</h3>
    <h4><span class="step">1</span> Create categories</h4>
    <p>Categories define what data you want your contributers to map. Say you want to create a map of the favourite places in London; then you would create a category for each restaurants, bar and great outdoor spots you like.</p>
    <p>Each category will have fields, that specify what information will be collected for that category. For example, a restaurant has a name, an address and the cuisine.</p>
    <p>Based on categories, the client application will create forms your contirbutors have to fill in order to add data to your project.</p>

    <h4><span class="step">2</span> Provide access to your data through data groupings</h4>
    <p>Data groupings provide users access to a subset of all data collected within a project. For each data group you create a set of filters to decide which data can be found in that data grouping. For example, you can create a grouping with all Italian restaurants that have a rating higher than 5.</p>

    <p>You can also select user groups, which will be allowed to access the data in a data grouping.</p>

    <h4><span class="step">3</span> Grant permissions to users with user groups.</h4>
    <p>Using user groups you can grant users permissions to your project. For each user group you can set whether its members can moderate, contribute or only read data. Furthermore, you can grant user groups access to the data groupings you created before.</p>
</div>