    <button type="button" class="close remove" aria-label="Close"><span aria-hidden="true">&times;</span></button>
    <h5>The contribution has been created</h5>

    <div class="form-group">
        <label for="created_at-min" class="col-sm-3 col-sm-offset-1 control-label">after</label>
        <div class="col-sm-7">
            <input data-date-format="YYYY-MM-DD H:mm" type="text" data-key="created_at" class="form-control datetime" name="created_at-min" id="created_at-min">
        </div>
    </div>
    <div class="form-group">
        <label for="created_at-max" class="col-sm-3 col-sm-offset-1 control-label">and before</label>
        <div class="col-sm-7">
            <input data-date-format="YYYY-MM-DD H:mm" type="text" data-key="created_at" class="form-control datetime" name="created_at-max" id="created_at-max">
        </div>
    </div>
