{% extends "base.html" %}

{% block title %} | Reset password{% endblock %}
{% block bodydata %} class="login" {% endblock %}

{% block main %}
        <div class="container">
            <div class="row">
                <div class="col-sm-8 col-sm-offset-2 col-md-6 col-md-offset-3">
                    <h3 class="header">Password reset successful</h3>
                    <div class="alert alert-success">
                        <p>We've emailed you instructions for setting your password. You should be receiving them shortly.</p>

                        <p>If you don't receive an email, please make sure you've entered the address you registered with, and check your spam folder.</p>
                    </div>
                </div>
            </div>
        </div>
{% endblock %}
