<div class="form-group">
    <label for="{{key}}-min" class="col-sm-3 col-sm-offset-1 control-label">is after</label>
    <div class="col-sm-7">
        <input data-date-format="YYYY-MM-DD" type="text" class="form-control date" data-key="{{key}}" name="{{key}}-min" id="{{key}}-min">
    </div>
</div>
<div class="form-group">
    <label for="{{key}}-max" class="col-sm-3 col-sm-offset-1 control-label">and before</label>
    <div class="col-sm-7">
        <input data-date-format="YYYY-MM-DD" type="text" class="form-control date" data-key="{{key}}" name="{{key}}-max" id="{{key}}-max">
    </div>
</div>
