<div class="field-filter form-horizontal">
{{> created-field}}
</div>
{{#each fields}}
    <div class="field-filter form-horizontal" data-key="{{key}}" data-type="{{fieldtype}}">
        {{#ifCond fieldtype 'TextField'}}
            {{> textfield}}
        {{/ifCond}}
        {{#ifCond fieldtype 'NumericField'}}
            {{> numericfield}}
        {{/ifCond}}
        {{#ifCond fieldtype 'DateTimeField'}}
            {{> datetimefield}}
        {{/ifCond}}
        {{#ifCond fieldtype 'DateField'}}
            {{> datefield}}
        {{/ifCond}}
        {{#ifCond fieldtype 'TimeField'}}
            {{> timefield}}
        {{/ifCond}}
        {{#ifCond fieldtype 'LookupField'}}
            {{> lookupfield}}
        {{/ifCond}}
        {{#ifCond fieldtype 'MultipleLookupField'}}
            {{> multiplelookupfield}}
        {{/ifCond}}
    </div>
{{/each}}
