{% extends "base.html" %}

{% load i18n %}
{% load account %}

{% block title %} | Confirm email{% endblock %}

{% block main %}
<div class="container">
    <div class="row">
        <div class="col-sm-8 col-sm-offset-2 col-md-6 col-md-offset-3">
            <h3 class="header">email_confirmed</h3>

            {% block content %}

            <h1>{% trans "Confirm E-mail Address" %}</h1>

            {% user_display confirmation.email_address.user as user_display %}

            <p>{% blocktrans with confirmation.email_address.email as email %}You have confirmed that <a href="mailto:{{email}}">{{ email }}</a> is an e-mail address for user {{ user_display }}.{% endblocktrans %}</p>

            {% endblock %}
        </div>
    </div>
</div>
{% endblock %}
